;;; Copyright 2023 Christine Lemmer-Webber
;;; Copyright 2024 Vivianne Langdon
;;;
;;; Licensed under the Apache License, Version 2.0 (the "License");
;;; you may not use this file except in compliance with the License.
;;; You may obtain a copy of the License at
;;;
;;;    http://www.apache.org/licenses/LICENSE-2.0
;;;
;;; Unless required by applicable law or agreed to in writing, software
;;; distributed under the License is distributed on an "AS IS" BASIS,
;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
;;; See the License for the specific language governing permissions and
;;; limitations under the License.

(define-module (bugafriend ncurses stuff)
  #:use-module (ncurses curses)
  #:use-module (rnrs enums)
  #:export (%YELLOW-N
            %GREEN-N
            %WHITE-N
            %RED-N
            %CYAN-N
            %MAGENTA-N

            install-colors!

            color-ref
            color-pair-ref

            screen-setup!))

;; Same order ncurses uses
(define colors
  '(black red green yellow blue magenta cyan white))
(define c-vec
  (vector COLOR_BLACK
          COLOR_RED
          COLOR_GREEN
          COLOR_YELLOW
          COLOR_BLUE
          COLOR_MAGENTA
          COLOR_CYAN
          COLOR_WHITE))

(define colors-len
  (length colors))
(define colors-enum
  (make-enumeration colors))

;; Get the id of a color (or a color against black)
(define color-ref
  (enum-set-indexer colors-enum))

(define (color-pair-ref fg-sym bg-sym)
  "Get the id of a color pair"
  (+ (color-ref fg-sym)
     (* (color-ref bg-sym) colors-len)))

(define %YELLOW-N (color-ref 'yellow))
(define %GREEN-N (color-ref 'green))
(define %WHITE-N (color-ref 'white))
(define %RED-N (color-ref 'red))
(define %CYAN-N (color-ref 'cyan))
(define %MAGENTA-N (color-ref 'magenta))

(define (do-init-pair! fg-color bg-color)
  (init-pair! (color-pair-ref fg-color bg-color)
              (vector-ref c-vec (color-ref fg-color))
              (vector-ref c-vec (color-ref bg-color))))

;; slow but whatevs, only done once
(define (install-colors!)
  "Install all colors into ncurses"
  (for-each (lambda (fg-color)
              (for-each (lambda (bg-color)
                          (do-init-pair! fg-color bg-color))
                        colors))
            colors))

(define* (screen-setup! #:optional (screen (initscr)))
  (noecho!)                             ; disable echoing characters
  (raw!)                                ; don't buffer input
  (keypad! screen #t)                   ; enable <f1>, arrow keys, etc
  ;; (start-color!)
                                        ; turn on colors
  (nodelay! screen #t)
  ;; (install-colors!)
                                        ; enable specific colors
  screen)
